var juxtaposition = d3.select("#juxtaposition").append("svg")
                                    .attr("width", width-20)
                                    .attr("height", 100);

function drawJuxtaRect(thisElementWidth, start, end, startHeight, endHeight, width, col){

    var startQuad = xoffset + Math.ceil(start);
    var endQuad = startQuad + Math.ceil(end*thisElementWidth+1);

    var quadPoints = (startQuad) + " " + (startHeight + width/2) + ", "
                        + (startQuad) + " " + (startHeight - width/2) + ", "
                        + (endQuad) + " " + (endHeight - width/2) + ", "
                        + (endQuad) + " " + (endHeight + width/2);

    juxtaposition.append("polygon")
        .attr("fill", col)
        .attr("points", quadPoints); 
}

function drawJuxtaArrow(n, i, thisElementWidth, start, gap, startHeight, endHeight, col){

    var arrowStart = start + xoffset;

    var length = Math.ceil(gap*thisElementWidth);
    var newLength = Math.ceil(Math.sqrt(Math.pow(length, 2) + Math.pow(endHeight-startHeight, 2)));
    var angleDeg = Math.atan2(endHeight - startHeight, gap*thisElementWidth) * 180 / Math.PI;
    var newColor = (highlighted[n] >= i-gap && highlighted[n] <= i) ? "#66FF00" : col;

    var height = startHeight;

    var size = (thisElementWidth+5 > 10) ? 10 : thisElementWidth+5;

    var triangleLength = (newLength > size) ? size : newLength;

    var arrowGroup = juxtaposition.append("g");

    arrowGroup.append("rect")
        .attr("height", size)
        .attr("width", newLength - triangleLength + 2)
        .attr("x", arrowStart)
        .attr("y", height-size/2)
        .style("fill", newColor);

    var trianglePoints = (arrowStart + newLength) + " " + (height) + ", " 
            + (arrowStart + newLength - triangleLength) + " " + (height + size) + ", " 
            + (arrowStart + newLength - triangleLength) + " " + (height - size) ; 

    arrowGroup.append("polygon")
              .attr("fill", newColor)
              .attr("points", trianglePoints);  

    var middle = (endHeight<startHeight) ? -((Math.abs(endHeight-startHeight))/2) : ((Math.abs(endHeight-startHeight))/2);

    arrowGroup.attr("transform", "rotate("+angleDeg+"," + (arrowStart+(length/2)) + "," + (startHeight+middle) + 
                    ") translate (0, " + (middle) + ")" );

    arrowGroup.on("click", function(){
      highlighted[n] = (highlighted[n] === i) ? 9999 : i;
      redrawD3();
      (highlighted[n] === 9999) ? (resetViewerSelection(n)) : highlightPV(n, alignToNotAlign[n][i]);
    });

}

function drawJuxtaHelix(n, j, thisElementWidth, start, gap, startHeight, endHeight, col){

  var startHelix = start + xoffset;
  var length = Math.ceil(gap*thisElementWidth);
  var height = startHeight;

  var newLength = Math.ceil(Math.sqrt(Math.pow(length, 2) + Math.pow(startHeight-endHeight, 2)));
  var angleDeg = Math.atan2(endHeight - startHeight, gap*thisElementWidth) * 180 / Math.PI;
  var newColor = (highlighted[n] >= j-gap && highlighted[n] <= j) ? "#66FF00" : col;

  var size = (thisElementWidth+9 > 20) ? 20 : thisElementWidth+9;

  var helixGroup = juxtaposition.append("g");
  
  var firstShade = (startHelix) + " " + (height - size/2) + ", "
                        + ((i+size/2>startHelix+newLength) ? startHelix+newLength : startHelix+size/2) + " " + (height - size/2) + ", "
                        + (startHelix) + " " + (height+size/2) + ", "
                        + (startHelix - size/4) + " " + (height + size/2) + ", "
                        + (startHelix - size/4) + " " + (height);

  helixGroup.append("polygon")
        .attr("fill", newColor)
        .attr("opacity", 0.3)
        .attr("points", firstShade);


  for (var i = startHelix; i < startHelix+newLength; i+=size) {
      var quadPoints = (i) + " " + (height - size/2) + ", "
                        + (i+size/2) + " " + (height - size/2) + ", "
                        + (i+size) + " " + (height + size/2) + ", "
                        + (i+size/2) + " " + (height + size/2);

      if (i+10 > startHelix+newLength) quadPoints = (i) + " " + (height - size/2) + ", "
                                                + (i+size/2) + " " + (height - size/2) + ", "
                                                + (i+3*size/4) + " " + (height) + ", "
                                                + (i+3*size/4) + " " + (height + size/2) + ", "
                                                + (i+size/2) + " " + (height+size/2);

  helixGroup.append("polygon")
          .attr("fill", newColor)
          .attr("points", quadPoints);    

  }

  for (var i = startHelix+size/2; i < startHelix+newLength; i+=size) {
      var quadPoints = (i) + " " + (height + size/2) + ", "
                        + (i+size/2) + " " + (height + size/2) + ", "
                        + (i+size) + " " + (height - size/2) + ", "
                        + (i+size/2) + " " + (height - size/2);

      if (i+10 > startHelix+newLength) quadPoints = (i) + " " + (height + size/2) + ", "
                                                + (i+size/2) + " " + (height + size/2) + ", "
                                                + (i+3*size/4) + " " + (height) + ", "
                                                + (i+3*size/4) + " " + (height - size/2) + ", "
                                                + (i+size/2) + " " + (height-size/2);

  helixGroup.append("polygon")
          .attr("fill", newColor)
          .attr("opacity", 0.5)
          .attr("points", quadPoints);    

  }

  var middle = (endHeight<startHeight) ? -((Math.abs(endHeight-startHeight))/2) : ((Math.abs(endHeight-startHeight))/2);

  helixGroup.attr("transform", "rotate("+angleDeg+"," + (startHelix+(length/2)) + "," + (startHeight+middle) + 
                  ") translate (0, " + (middle) + ")" );

  helixGroup.on("click", function(){
      highlighted[n] = (highlighted[n] === j) ? 9999 : j;
      redrawD3();
      (highlighted[n] === 9999) ? (resetViewerSelection(n)) : highlightPV(n, alignToNotAlign[n][j]);
    });
}

function drawJuxtaposition(n){

  var radialGradient = juxtaposition.append("defs")
    .append("radialGradient")
      .attr("id", "radial-gradient");

  radialGradient.append("stop")
      .attr("offset", "0%")
      .attr("stop-color", (n===reference) ? colorMolecule[n] : "gray");

  radialGradient.append("stop")
      .attr("offset", "100%")
      .attr("stop-color", "#fff");

  juxtaposition.append("circle")
          .attr("cx", xoffset/2)
          .attr("cy", n*45 + 45)
          .attr("r", 5)
          .attr("fill", "url(#radial-gradient)")//BallColors[n][0])
          .on("click", function(d) {
            var chain = Math.round((this.cy.animVal.value - 25)/45);
            displayMessage("Selected " + proteins[chain] + " as reference chain.");
            if(chain !== reference) {
              reference = chain;
              redrawD3();
            }
          });

  var startAA = juxtaZoom[n][0];
  var endAA = juxtaZoom[n][1];
  var thisElementWidth = (width-100)/(endAA-startAA);
  
  if(juxtaElementSet) {
    juxtaElementWidth = thisElementWidth;
    juxtaElementSet = false;
  }

  var globalI = 0;

  if(n !== reference) {
    juxtaposition.append("line")
            .attr('x1', xoffset)
            .attr('y1', 45 + n*45)
            .attr('x2', thisElementWidth*(endAA-startAA))
            .attr('y2', 45 + n*45)
            .style('stroke', 'lightgray')
            .style('stroke-width', 1)
  }

  var lastHeight = 0;

  for (var i = startAA; i < endAA - 1; i++) {
      var ss = alignedSS[n].charAt(i);
      var initHeight = 45 + n*45;
      var endHeight = 45 + n*45;

      var start = i;
      while (alignedSS[n].charAt(i+1) === ss) i++;
      var gap = i - start + 1;

      if(n!==reference && (ss == 'H' || ss == 'E')) {
        if(alignedSS[reference].charAt(start) == '-') {
          initHeight -= 15;
        } else {
          //console.log(atomDiffs[n][start]);
          initHeight -= (15 < atomDiffs[n][start])? 15 : atomDiffs[n][start];
        } 
        if(alignedSS[reference].charAt(i) == '-') {
          endHeight -= 15;
          lastHeight = endHeight;
        } else {
          endHeight -= (15 < atomDiffs[n][i]) ? 15 : atomDiffs[n][i];
          lastHeight = endHeight;
        }
      }

      if(n!==reference && ss == 'C') {
          if(lastHeight !== 0) initHeight = lastHeight;
          if (alignedSS[n].charAt(i+1) === 'H' || alignedSS[n].charAt(i+1) === 'E') {
            if(alignedSS[reference].charAt(i+1) == '-') {
              endHeight -= 15;
            } else {
              endHeight -= (15 < atomDiffs[n][i+1]) ? 15 : atomDiffs[n][i+1];
            }
          } else {
            lastHeight = 0;
          }
      }

      if(n!==reference && ss == '-') lastHeight = 0;

      if (ss === 'E') drawJuxtaArrow(n, i, thisElementWidth, globalI, gap, initHeight, endHeight, colorMolecule[n]);
      if (ss === 'H') drawJuxtaHelix(n, i, thisElementWidth, globalI, gap, initHeight, endHeight, colorMolecule[n]);
      if (ss === '-') drawJuxtaRect(thisElementWidth, globalI,  gap, initHeight, endHeight, 3, "lightgray");
      if (ss === 'C') drawJuxtaRect(thisElementWidth, globalI,  gap, initHeight, endHeight, 3, colorMolecule[n]);

      globalI += Math.round( gap * thisElementWidth);
  }

    //selection
    function rect(x, y, w, h) {
      return "M"+[x,y]+" l"+[w,0]+" l"+[0,h]+" l"+[-w,0]+"z";
    }

    var selection = juxtaposition.append("path")
      .attr("class", "selection")
      .attr("visibility", "hidden");

    var startSelection = function(start) {
        selection.attr("d", rect(start[0], start[0], 0, 0))
          .attr("visibility", "visible");
    };

    function inBounderiesX(pos, move){
      var newPos = pos;
      if(pos < 0) newPos = 0; 
      if (pos > juxtaposition.attr("width")) newPos = juxtaposition.attr("width");
      return newPos - move;
    }

    function inBounderiesY(pos, move){
      var newPos = pos;
      if(pos < 0) newPos = 0; 
      if (pos > juxtaposition.attr("height")) newPos = juxtaposition.attr("height");
      return newPos - move;
    }

    var moveSelection = function(start, moved) {
        selection.attr("d", rect(start[0], start[1], inBounderiesX(moved[0], start[0]), inBounderiesY(moved[1], start[1])));
    };

    var endSelection = function(start, end) {
      selection.attr("visibility", "hidden");
    };

    juxtaposition.on("mousedown", function() {
      var subject = d3.select(window), parent = this.parentNode,
          start = d3.mouse(parent);
        startSelection(start);
        subject
          .on("mousemove.selection", function() {
            moveSelection(start, d3.mouse(parent));
          }).on("mouseup.selection", function() {
            endSelection(start, d3.mouse(parent));

              //if the chain is not zoomed, zoom to the section          
              var beginChain = Math.floor((((start[1]<d3.mouse(parent)[1]) ? start[1] : d3.mouse(parent)[1]))/45);
              if(beginChain < 0) beginChain = 0;
              var endChain = Math.floor((((start[1]>d3.mouse(parent)[1]) ? start[1] : d3.mouse(parent)[1]) - 45 )/45);
              if(endChain > proteins.length-1) endChain = proteins.length-1;
              
              var beginRange = Math.floor((((start[0]<d3.mouse(parent)[0]) ? start[0] : d3.mouse(parent)[0]) - 20)/juxtaElementWidth);
              if(beginRange < 0) beginRange = 0;
              var endRange = Math.floor((((start[0]>d3.mouse(parent)[0]) ? start[0] : d3.mouse(parent)[0]) - 20)/juxtaElementWidth);
              if(endRange > alignedSS[n].length) endRange = alignedSS[n].length;
              
              if((endRange - beginRange)*elementWidth < 24) {
                //do nothing for such a small zoom
                //console.log("fuck it.");
                displayMessage("Zoom selection is too narrow.");
                setTimeout(function () {
                    displayMessage(getProteinNames());
                }, 2000);
              } else {
                displayMessage(getProteinNames());
                // console.log("Chains: " + beginChain + ", "  + endChain);
                // console.log("Range: " + beginRange + ", "  + endRange);
                for (var i = beginChain; i < endChain + 1; i++) {
                  if(juxtaZoom[i][0] === 0 && juxtaZoom[i][1] === alignedSS[i].length) juxtaZoom[i] = [beginRange, endRange];
                }
                redrawD3();
              }
            // console.log(start);
            // console.log(d3.mouse(parent));
            subject.on("mousemove.selection", null).on("mouseup.selection", null);
          });
    });

}